﻿using System;
using System.Collections.Generic;
using NXOpen;
using NXOpen.UF;
using NXOpen.UserDefinedObjects;
using NXOpen.VectorArithmetic;

public class RealtimeTracer : BehaviorDef
{
    // nx env
    static Session theSession;
    static UFSession theUFSession;
    static Part theWorkPart;
    static BasePart myBasePart = null;
    
    static UserDefinedClass myUDOclass = null;

    // physics
    static RigidBody m_rigidBody = null;
    static SignalAdapter m_signalAdapter = null;

    // line info
    static int color = 186;
    static Queue<Vector3> samplePoints = new Queue<Vector3>();
    static List<double> drawPoints = new List<double>();
    static Vector3 start3;
    static Vector3 end3;
    double length = 0.0;
    //
    NXOpen.Point assPoint;
    public override void Define(IDefinitionContext access)
    {
        // Insert definition code here
        access.Connect("Rigid Body", out m_rigidBody);
        access.Connect("Signal Adapter", out m_signalAdapter);
    }

    public override void Start(IRuntimeContext context)
    {
        // Insert start up code here
        theSession = Session.GetSession();
        theUFSession = UFSession.GetUFSession();
        myBasePart = theSession.Parts.BaseDisplay;
        theWorkPart = theSession.Parts.Work;
        if (!Check(context))
            context.Error(true, "The modified tracer part is not acceptable, !USE ORIGINAL TRACER PART!");
        assPoint = theWorkPart.Points.CreatePoint(new Point3d(0.0, 0.0, 0.0));
        initUDO();
    }
    public override void Stop(IRuntimeContext context)
    {
        // Insert stopping code here
        if (theUFSession != null)
            theUFSession.Obj.DeleteObject(assPoint.Tag);
        myUDOclass.Dispose();
    }
    public override void Step(IRuntimeContext context, double dt)
    {
        // NXOPEN APIs can not be called in Step function since Step is running in the worker thread. 
        // only MCD runtime APIs can be called in the Step function
        // Insert simulation step code here
        if (m_signalAdapter.Signals[0].BoolValue)
        {
            samplePoints.Enqueue(m_rigidBody.Position);
            if (m_signalAdapter.Signals[1].IntValue >= 1 && m_signalAdapter.Signals[1].IntValue <= 216)
                color = m_signalAdapter.Signals[1].IntValue;
        }
        else
        {
            samplePoints.Clear();
        }
    }
    public override void Refresh(IRuntimeContext context)
    {
        // Insert simulation refreshing code here. This is called when it is safe
        // to read and change runtime variables outside of the main control loop.
        NewMethod(color);
        //if (m_signalAdapter.Signals[1].BoolValue)
        //{
        //    Point3d point3 = new Point3d(
        //     m_signalAdapter.Signals[2].FloatValue * 1000.0,
        //     m_signalAdapter.Signals[3].FloatValue * 1000.0,
        //     m_signalAdapter.Signals[4].FloatValue * 1000.0);
        //    assPoint.SetCoordinates(point3);
        //    m_signalAdapter.Signals[1].BoolValue = false;
        //}
    }
    public override void Repaint()
    {
        // Insert simulation repainting code here. This is where you use the data
        // taken from the runtime to update the display.
    }
    private bool Check(IRuntimeContext context)
    {
        if (m_signalAdapter.Signals.Length < 2)
            return false;
        //
        ComponentPart part = m_rigidBody.AskAssembly();
        if (part.NumParts != 6)
            return false;
        foreach (RuntimeObject runtimeObject in part.Parts)
        {
            ProxyObject m_proxyObject = runtimeObject as ProxyObject;
            {
                if (m_proxyObject != null)
                {
                    Tag tag = m_proxyObject.GetPhysicsObject();
                    NXOpen.Mechatronics.ProxyObject proxyObject = (NXOpen.Mechatronics.ProxyObject)NXOpen.Utilities.NXObjectManager.Get(tag);
                    NXOpen.Mechatronics.ProxyObjectBuilder proxyObjectBuilder = theWorkPart.ProxyObjectCollection.CreateProxyObjectBuilder(proxyObject);
                    NXOpen.Mechatronics.ProxyObjectParameterBuilderList proxyObjectParameterBuilderList;
                    proxyObjectParameterBuilderList = proxyObjectBuilder.ParameterList;
                    if (proxyObjectParameterBuilderList.Length < 2)
                        return false;
                    string name1 = proxyObjectParameterBuilderList.FindItem(0).AttrName;
                    string name2 = proxyObjectParameterBuilderList.FindItem(1).AttrName;
                    proxyObjectBuilder.Destroy();
                    if (name1 == "LLShen" && name2 == "NXMCD机电一体化")
                    {
                        return true;
                    }
                }
            }
        }
        
        return false;
    }
    private void NewMethod(int theColor)
    {
        if (samplePoints.Count < 2)
        {
            return;
        }
        
        drawPoints.Clear();
        while (samplePoints.Count > 1)
        {
            start3 = samplePoints.Dequeue();
            end3 = samplePoints.Peek();
            Vector3 vector3 = end3 - start3;
            length = Math.Sqrt(vector3.x * vector3.x + vector3.y * vector3.y + vector3.z * vector3.z) * 1000;

            while (length < 1 && samplePoints.Count > 1)
            {
                samplePoints.Dequeue();
                end3 = samplePoints.Peek();
                Vector3 vector4 = end3 - start3;
                length = Math.Sqrt(vector3.x * vector3.x + vector3.y * vector3.y + vector3.z * vector3.z) * 1000;
            }
            drawPoints.Add(start3.x * 1000);
            drawPoints.Add(start3.y * 1000);
            drawPoints.Add(start3.z * 1000);
            drawPoints.Add(end3.x * 1000);
            drawPoints.Add(end3.y * 1000);
            drawPoints.Add(end3.z * 1000);
        }

        UserDefinedObject theUDO = myBasePart.UserDefinedObjectManager.CreateUserDefinedObject(myUDOclass);
        theUDO.Color = theColor;
        theUDO.SetDoubles(drawPoints.ToArray());
        UserDefinedObject.LinkDefinition[] links = new UserDefinedObject.LinkDefinition[1];
        links[0].AssociatedObject = assPoint;
        theUDO.SetLinks(UserDefinedObject.LinkType.Type1, links);
        theUDO.ClearUserDefinedObjectStatus();
        theUDO.RedisplayObject();
    }
    void initUDO()
    {
        if (theSession == null) return;

        try // In case the Class already exists
        {
            myUDOclass = theSession.UserDefinedClassManager.
                GetUserDefinedClassFromClassName("MCD_RealtimeTracer_UDO");
            //theSession.LogFile.WriteLine("Modifying behavior of existing " + myUDOclass.FriendlyName);
        }
        catch  // First time this session
        {
            myUDOclass = theSession.UserDefinedClassManager.CreateUserDefinedObjectClass
                ("MCD_RealtimeTracer_UDO", "MCD RealtimeTracer UDO");
            //theSession.LogFile.WriteLine("Registered " + myUDOclass.FriendlyName);
        }

        myUDOclass.AllowQueryClassFromName = UserDefinedClass.AllowQueryClass.On;
        UI.GetUI().SelectionManager.SetSelectionStatusOfUserDefinedClass(myUDOclass, false);

        myUDOclass.AddAttentionPointHandler(new UserDefinedClass.DisplayCallback(
            theDisplayCB));
        myUDOclass.AddDisplayHandler(new UserDefinedClass.DisplayCallback(
           theDisplayCB));
        myUDOclass.AddFitHandler(new UserDefinedClass.DisplayCallback(
            theDisplayCB));
        myUDOclass.AddSelectionHandler(new UserDefinedClass.DisplayCallback(
            theDisplayCB));
        myUDOclass.AddScreenSizeFitHandler(new UserDefinedClass.DisplayCallback(
            theDisplayCB));
        //myUDOclass.AddEditHandler(new UserDefinedClass.GenericCallback(
        //    myEditCB));
    }
    int theDisplayCB(UserDefinedDisplayEvent thisEvent)
    {
        try
        {
            UserDefinedObject.LinkDefinition[] theLink =
                thisEvent.UserDefinedObject.GetLinks(UserDefinedObject.LinkType.Type1);
            if (theLink.Length < 1)
                return 0;
            NXOpen.Point point = (NXOpen.Point)theLink[0].AssociatedObject;

            double[] tDs = thisEvent.UserDefinedObject.GetDoubles();
            List<Point3d> point3Ds = new List<Point3d>();
            for (int i = 0; i < tDs.Length / 3; i++)
            {
                //point3Ds.Add(new Point3d(point.Coordinates.X + tDs[i * 3], point.Coordinates.Y + tDs[i * 3 + 1], point.Coordinates.Z + tDs[i * 3 + 2]));
                point3Ds.Add(new Point3d(tDs[i * 3], tDs[i * 3 + 1], tDs[i * 3 + 2]));
            }
            thisEvent.DisplayContext.DisplayPolyline(point3Ds.ToArray());
        }
        catch (NXOpen.NXException ex)
        {
            //Echo("theDisplayCB threw error " + ex.ErrorCode + ": " + ex.Message);
        }
        return 0;
    }
}
